<?php
/**
 *
 * Plugin Main Class
 *
 * This class is responsible for initializing the admin side of the plugin.
 *
 * @package Maintainer
 * @subpackage Admin
 * @since 1.0.0
 */

namespace Maintainer\Admin;

use Maintainer\Core\Helpers;

class Init {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The Instance
	 *
	 * @since 1.0.0
	 * @access   protected
	 * @var boolean
	 */
	private static $_instance = null;

	/**
	 * Initialize the class and set its properties.
	 */
	public function __construct() {
		$this->initializePlugin();
		$this->setupAdminActions();
	}

	/**
	 * Instance of the class.
	 *
	 * @return void
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
	}

	/**
	 * Initialize the plugin.
	 *
	 * @return void
	 */
	private function initializePlugin() {
		$this->version     = defined( 'MAINTAINER_VERSION' ) ? MAINTAINER_VERSION : '1.0.0';
		$this->plugin_name = defined( 'MAINTAINER_NAME' ) ? MAINTAINER_NAME : 'maintainer';
	}

	/**
	 * Setup the admin actions.
	 *
	 * @return void
	 */
	private function setupAdminActions() {
		add_action( 'admin_menu', array( $this, 'addPluginAdminMenu' ) );
		add_filter( 'plugin_action_links_' . MAINTAINER_PLUGIN_BASE, array( $this, 'addActionLinks' ), 10, 1 );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueueStyles' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueueScripts' ) );
		add_action( 'wp_ajax_maintainer_save_settings', array( $this, 'saveSettings' ) );
		add_action( 'admin_bar_menu', array( $this, 'addMaintenanceToolbarMenu' ), 100 );
	}

	/**
	 * Enqueue the admin styles.
	 *
	 * @return void
	 */
	public function enqueueStyles() {
		wp_enqueue_style( $this->plugin_name, MAINTAINER_PLUGIN_URL . 'assets/css/admin.css', array(), $this->version, 'all' );

		// Add custom inline CSS for the maintenance mode indicator
		$custom_css = '
			#wp-admin-bar-maintenance-mode .ab-item {
				display: inline-flex!important;
				align-items: center!important;
			}
			#wp-admin-bar-maintenance-mode.maintenance-active .ab-item {
				background-color: #047857 !important;
				color: #fff !important;
				display: inline-flex;
				align-items: center;
			}
			#wp-admin-bar-maintenance-mode .ab-item:hover {
				background-color: #14b8a6 !important;
				color: #fff !important;
			}
		';
		wp_add_inline_style( $this->plugin_name, $custom_css );
	}

	/**
	 * Enqueue the admin scripts.
	 *
	 * @return void
	 */
	public function enqueueScripts() {
		if ( ! $this->isPluginAdmin() ) {
			return;
		}

		wp_enqueue_script( $this->plugin_name, MAINTAINER_PLUGIN_URL . 'assets/js/admin.js', array(), $this->version, true );

		// Add admin ajax URL and nonce
		wp_localize_script(
			$this->plugin_name,
			'maintainerAdmin',
			array(
				'ajaxurl' => admin_url( 'admin-ajax.php' ),
				'nonce'   => wp_create_nonce( 'maintainer_settings_save' ),
			)
		);
	}

	/**
	 * Check if the current screen is the plugin admin.
	 *
	 * @return bool
	 */
	public function isPluginAdmin() {
		return str_contains( get_current_screen()->base, 'maintainer' );
	}

	/**
	 * Add the plugin admin menu.
	 *
	 * @return void
	 */
	public function addPluginAdminMenu() {
		$this->addMenuPage( 'Maintainer', 'maintainer', 'displayPluginSetupPage' );
	}

	/**
	 * Add a menu page.
	 *
	 * @param string $title
	 * @param string $slug
	 * @param string $template
	 * @param string $type
	 * @param string $sub
	 * @param string $priority
	 * @return void
	 */
	public function addMenuPage( $title, $slug, $template, $type = 'page', $sub = 'maintainer', $priority = '99' ) {
		$menuFunction = $this->getMenuFunction( $type );
		$menuFunction( $title, $title, 'manage_options', $slug, array( $this, $template ), $priority );
	}

	/**
	 * Get the menu function.
	 *
	 * @param string $type
	 * @return string
	 */
	private function getMenuFunction( $type ) {
		$menuFunctions = array(
			'page' => 'add_options_page',
			'sub'  => 'add_submenu_page',
			'menu' => 'add_menu_page',
		);
		return $menuFunctions[ $type ] ?? 'add_menu_page';
	}

	/**
	 * Add action links.
	 *
	 * @param array $links Existing plugin action links
	 * @return array Modified plugin action links
	 */
	public function addActionLinks( $links ) {
		$additional_links = array(
			'settings' => '<a href="' . admin_url( 'options-general.php?page=' . $this->plugin_name ) . '">' . __( 'Settings', 'maintainer' ) . '</a>',
		);

		return array_merge( $links, $additional_links );
	}

	/**
	 * Display the plugin setup page.
	 *
	 * @return void
	 */
	public function displayPluginSetupPage() {
		Helpers::get_template( 'settings', 'admin' );
	}

	public function saveSettings() {
		$response = array(
			'success' => false,
			'message' => __( 'An error occurred while saving settings.', 'maintainer' ),
		);

		// Verify nonce
		if ( ! check_ajax_referer( 'maintainer_settings_save', 'security', false ) ) {
			$response['message'] = __( 'Invalid security token.', 'maintainer' );
			wp_send_json( $response );
		}

		// Check user permissions
		if ( ! current_user_can( 'manage_options' ) ) {
			$response['message'] = __( 'You do not have permission to perform this action.', 'maintainer' );
			wp_send_json( $response );
		}

		// Update options with proper sanitization and validation
		$checkbox_fields = array(
			'maintainer_active'  => 'maintainer-active',
			'maintainer_noindex' => 'maintainer-noindex',
		);

		foreach ( $checkbox_fields as $option => $field ) {
			update_option( $option, isset( $_POST[ $field ] ) ? true : false );
		}

		// Text fields
		$text_fields = array(
			'maintainer_primary_text'     => 'maintainer-primary-text',
			'maintainer_page_title'       => 'maintainer-page-title',
			'maintainer_page_description' => 'maintainer-page-description',
			'maintainer_background_color' => 'maintainer-background-color',
			'maintainer_text_color'       => 'maintainer-text-color',
			'maintainer_query'            => 'maintainer-query',
			'maintainer_template'         => 'maintainer-template',
		);

		foreach ( $text_fields as $option => $field ) {
			if ( isset( $_POST[ $field ] ) ) {
				update_option( $option, sanitize_text_field( wp_unslash( $_POST[ $field ] ) ) );
			}
		}

		// Handle secondary text separately to allow HTML
		if ( isset( $_POST['maintainer-secondary-text'] ) ) {
			$allowed_html = wp_kses_allowed_html( 'post' );
			update_option(
				'maintainer_secondary_text',
				wp_kses( wp_unslash( $_POST['maintainer-secondary-text'] ), $allowed_html )
			);
		}

		// IP addresses
		if ( isset( $_POST['maintainer-ips'] ) ) {
			// Sanitize and validate the IP addresses
			$valid_ips = array();
			$raw_ips   = sanitize_textarea_field( wp_unslash( $_POST['maintainer-ips'] ) );

			// Split into array and process each IP
			$ip_candidates = array_filter( array_map( 'trim', explode( "\n", $raw_ips ) ) );

			foreach ( $ip_candidates as $ip ) {
				// Remove any potential harmful characters
				$clean_ip = preg_replace( '/[^0-9a-fA-F:.]/', '', $ip );

				// Validate IPv4 and IPv6 addresses
				if ( filter_var( $clean_ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE ) ) {
					$valid_ips[] = $clean_ip;
				}
			}

			update_option( 'maintainer_ips', $valid_ips );
		}

		// Roles
		if ( isset( $_POST['maintainer-allowed-roles'] ) && is_array( $_POST['maintainer-allowed-roles'] ) ) {
			$roles = array_map(
				'sanitize_text_field',
				wp_unslash( $_POST['maintainer-allowed-roles'] )
			);
			update_option( 'maintainer_roles', $roles );
		} else {
			update_option( 'maintainer_roles', array() );
		}

		$response['success'] = true;
		$response['message'] = __( 'Settings saved successfully!', 'maintainer' );

		wp_send_json( $response );
	}

	/**
	 * Add Maintenance menu to the admin toolbar
	 *
	 * @param \WP_Admin_Bar $admin_bar
	 * @return void
	 */
	public function addMaintenanceToolbarMenu( $admin_bar ) {
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		$is_active   = get_option( 'maintainer_active', false );
		$status_text = $is_active
			? __( 'Maintenance Active', 'maintainer' )
			: __( 'Maintenance Inactive', 'maintainer' );

		// SVG icon for maintenance
		$icon = '<span class="maintenance-icon" style="margin-right: 3px;">' .
			'<svg style="width: 16px; height: 16px; vertical-align: middle;" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24">
					<path fill="currentColor" d="m10.68 2.105l-6.61 3.8l-.002.002A2.65 2.65 0 0 0 2.75 8.198v7.603a2.64 2.64 0 0 0 1.318 2.292l.003.002l6.608 3.799h.002a2.63 2.63 0 0 0 2.639 0h.001l6.608-3.8h.003A2.65 2.65 0 0 0 21.25 15.8V8.2a2.65 2.65 0 0 0-1.318-2.292l-6.61-3.8l-.002-.002a2.63 2.63 0 0 0-2.64 0M8.75 12a3.25 3.25 0 1 1 6.5 0a3.25 3.25 0 0 1-6.5 0" />
				</svg>' .
			'</span>';

		$admin_bar->add_node(
			array(
				'id'     => 'maintenance-mode',
				'title'  => $icon . $status_text,
				'href'   => admin_url( 'options-general.php?page=maintainer' ),
				'meta'   => array(
					'class' => $is_active ? 'maintenance-active' : '',
				),
				'parent' => 'top-secondary',
			)
		);
	}
}