<?php

namespace Maintainer\Frontend;

class Init {
	/**
	 * The ID of this plugin.
	 *
	 * @since 1.0.0
	 * @var string
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since 1.0.0
	 * @var string
	 */
	private $version;

	/**
	 * The Instance
	 *
	 * @since 1.0.0
	 * @var boolean
	 */
	private static $_instance = null;

	public function __construct() {
		$this->setPluginDetails();
		$this->loadStylesAndScripts();
		add_action( 'template_redirect', array( $this, 'addMaintenancePage' ), 999 );
	}

	/**
	 * Instance.
	 *
	 * @return void
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
	}

	/**
	 * Set plugin details.
	 *
	 * @return void
	 */
	private function setPluginDetails() {
		$this->version     = defined( 'MAINTAINER_VERSION' ) ? MAINTAINER_VERSION : '1.0.0';
		$this->plugin_name = defined( 'MAINTAINER_NAME' ) ? MAINTAINER_NAME : 'maintainer';
	}

	/**
	 * Load styles and scripts.
	 *
	 * @return void
	 */
	private function loadStylesAndScripts() {
		// Add our styles
		add_action( 'wp_enqueue_scripts', array( $this, 'styles' ) );

		// Remove other styles when maintenance mode is active
		add_action( 'wp_enqueue_scripts', array( $this, 'dequeueOtherStyles' ), 9999 );
	}

	/**
	 * Dequeue all styles except our maintenance styles.
	 *
	 * @return void
	 */
	public function dequeueOtherStyles() {
		// Only dequeue if maintenance mode is active and user should see maintenance page
		$maintainer_active = get_option( 'maintainer_active' );
		$maintainer_query  = get_option( 'maintainer_query' );
		$maintainer_roles  = get_option( 'maintainer_roles', array() );
		$maintainer_ips    = get_option( 'maintainer_ips', array() );

		// Exit early if maintenance mode is not active
		if ( ! $maintainer_active ) {
			return;
		}

		// Check if user has allowed role
		$user_has_allowed_role = false;
		if ( is_user_logged_in() && ! empty( $maintainer_roles ) ) {
			$user = wp_get_current_user();
			foreach ( $maintainer_roles as $role ) {
				if ( in_array( $role, (array) $user->roles ) ) {
					return; // Exit if user has allowed role
				}
			}
		}

		// Check if user has allowed IP
		if ( ! empty( $maintainer_ips ) ) {
			$current_ip = $this->getClientIP();
			foreach ( $maintainer_ips as $allowed_ip ) {
				if ( trim( $allowed_ip ) === $current_ip ) {
					return; // Exit if user has allowed IP
				}
			}
		}

		// Check for preview parameter with nonce verification
		if ( isset( $_GET['mp'], $_GET['_wpnonce'] ) &&
			 wp_verify_nonce( sanitize_key( $_GET['_wpnonce'] ), 'maintainer_preview' ) &&
			 $_GET['mp'] === $maintainer_query
		) {
			return; // Exit if correct preview parameter is present and nonce is valid
		}

		// If we get here, user should see maintenance page, so dequeue other styles
		global $wp_styles;

		// Store all registered styles
		$registered_styles = $wp_styles->registered;

		// Dequeue all styles except our maintenance styles
		foreach ( $registered_styles as $handle => $style ) {
			// Skip our maintenance styles
			if ( $handle === $this->plugin_name ) {
				continue;
			}

			wp_dequeue_style( $handle );
			wp_deregister_style( $handle );
		}
	}

	/**
	 * Load styles.
	 *
	 * @return void
	 */
	public function styles() {
		// Only enqueue styles if maintenance mode is active and user should see maintenance page
		$maintainer_active = get_option('maintainer_active');
		$maintainer_query = get_option('maintainer_query');
		$maintainer_roles = get_option('maintainer_roles', array());
		$maintainer_ips = get_option('maintainer_ips', array());

		// Exit early if maintenance mode is not active
		if (!$maintainer_active) {
			return;
		}

		// Check if user has allowed role
		if (is_user_logged_in() && !empty($maintainer_roles)) {
			$user = wp_get_current_user();
			foreach ($maintainer_roles as $role) {
				if (in_array($role, (array) $user->roles)) {
					return;
				}
			}
		}

		// Check if user has allowed IP
		if (!empty($maintainer_ips)) {
			$current_ip = $this->getClientIP();
			foreach ($maintainer_ips as $allowed_ip) {
				if (trim($allowed_ip) === $current_ip) {
					return;
				}
			}
		}

		// Check for preview parameter
		if (isset($_GET['mp'], $_GET['_wpnonce']) &&
			wp_verify_nonce(sanitize_key($_GET['_wpnonce']), 'maintainer_preview') &&
			$_GET['mp'] === $maintainer_query
		) {
			return;
		}

		// If we get here, enqueue maintenance styles
		wp_enqueue_style(
			$this->plugin_name,
			MAINTAINER_PLUGIN_URL . 'assets/css/public.css',
			array(),
			$this->version,
			'all'
		);

		// Define inline styles
		$inline_styles = '
			body, html {
				font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";
				overflow: hidden;
				height: 100%;
				width: 100%;
				margin: 0;
			}

			.container {
				display: flex;
				justify-content: center;
				align-items: center;
				height: 100%;
				width: 100%;
				flex-direction: column;
			}
		';

		// Add inline styles
		wp_add_inline_style($this->plugin_name, $inline_styles);
	}

	public function addMaintenancePage() {
		// Verify nonce for any form submissions
		if ( ! empty( $_POST ) ) {
			check_admin_referer( 'maintainer_action', 'maintainer_nonce' );
		}

		// Get maintenance settings
		$maintainer_active = get_option( 'maintainer_active' );
		$maintainer_query  = get_option( 'maintainer_query' );
		$maintainer_roles  = get_option( 'maintainer_roles', array() );
		$maintainer_ips    = get_option( 'maintainer_ips', array() );

		// Get and sanitize the 'm' parameter from URL
		$preview_param = '';
		if ( isset( $_GET['mp'] ) ) {
			$preview_param = sanitize_text_field( wp_unslash( $_GET['mp'] ) );
		}

		// Check if user has any allowed role
		$user_has_allowed_role = false;
		if ( is_user_logged_in() && ! empty( $maintainer_roles ) ) {
			$user = wp_get_current_user();
			foreach ( $maintainer_roles as $role ) {
				if ( in_array( $role, (array) $user->roles ) ) {
					$user_has_allowed_role = true;
					break;
				}
			}
		}

		// Check if current IP is allowed
		$user_has_allowed_ip = false;
		if ( ! empty( $maintainer_ips ) ) {
			$current_ip = $this->getClientIP();

			if ( $current_ip !== '0.0.0.0' ) {
				foreach ( $maintainer_ips as $allowed_ip ) {
					$allowed_ip = trim( sanitize_text_field( $allowed_ip ) );
					if ( $current_ip === $allowed_ip ) {
						$user_has_allowed_ip = true;
						break;
					}
				}
			}
		}

		// Show maintenance page if:
		// 1. Maintenance mode is active AND
		// 2. User is not logged in OR doesn't have allowed role AND
		// 3. User IP is not in allowed list AND
		// 4. URL does NOT have the correct preview parameter
		if ( $maintainer_active &&
			( ! is_user_logged_in() || ! $user_has_allowed_role ) &&
			! $user_has_allowed_ip &&
			( ! $maintainer_query || $preview_param !== $maintainer_query )
		) {
			// Get template path
			$template_path = MAINTAINER_PLUGIN_DIR . 'templates/public/maintenance.php';

			// Check if template exists
			if ( file_exists( $template_path ) ) {
				include $template_path;
				exit();
			}
		}
	}

	private function getClientIP() {
		$ip_headers = array(
			'HTTP_CLIENT_IP',
			'HTTP_X_FORWARDED_FOR',
			'HTTP_X_FORWARDED',
			'HTTP_X_CLUSTER_CLIENT_IP',
			'HTTP_FORWARDED_FOR',
			'HTTP_FORWARDED',
			'REMOTE_ADDR',
		);

		foreach ( $ip_headers as $header ) {
			// Sanitize the header name first
			$header = sanitize_key( $header );

			if ( isset( $_SERVER[ $header ] ) ) {
				// Double sanitization: first unslash then sanitize
				$raw_ip = sanitize_text_field( wp_unslash( $_SERVER[ $header ] ) );

				// Handle X-Forwarded-For header which may contain multiple IPs
				if ( $header === 'HTTP_X_FORWARDED_FOR' ) {
					$ips    = array_map( 'trim', explode( ',', $raw_ip ) );
					$raw_ip = isset( $ips[0] ) ? $ips[0] : '';
				}

				// Remove any non-IP characters
				$clean_ip = preg_replace( '/[^0-9a-fA-F:.]/', '', $raw_ip );

				// Validate IP format and exclude private/reserved ranges
				if ( ! empty( $clean_ip ) &&
					filter_var(
						$clean_ip,
						FILTER_VALIDATE_IP,
						FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE
					)
				) {
					return $clean_ip;
				}
			}
		}

		// If no valid IP is found, return a safe default
		return '0.0.0.0';
	}
}